package sys;

import android.app.KeyguardManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.res.AssetManager;
import android.content.res.Configuration;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.util.Log;
import android.view.MotionEvent;
import android.view.WindowManager;
import android.widget.FrameLayout;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.egl.EGLDisplay;
import javax.microedition.khronos.opengles.GL10;

import sys.SoundManager;


/********************
    アクティビティ
 ********************/
public class BaseActivity extends FragmentActivity
{
	static {
		System.loadLibrary("native");
    }


	public final static int		KEY_BACK	= 1;		// バックキー
	public final static int		KEY_YES		= 2;		// ダイアログ用
	public final static int		KEY_NO		= 3;


	protected static FrameLayout	base_layout;				// ベースレイアウト
	private static BaseView			surface_view;				// ビュー
	private static BaseRenderer		renderer;					// レンダラー
	private static int				phase;						// 実行段階
	private static int				orientation;				// 画面の向き
	protected static int			screen_width;				// 画面の大きさ
	protected static int			screen_height;

	private static ScheduledExecutorService		executor;		// 定期実行管理
	private static ScheduledFuture<?>			future;

	private static short[]		touch_status = new short[5*3];	// タッチパネル状態
	protected static int		key_status;						// キー入力状態

	public native int		initNative(boolean _init, int _w, int _h, AssetManager _mgr);	// native部初期化
	public native void		quitNative();													// native部終了
	public native void		pauseNative();													// native部一時停止
	public native boolean	updateNative(short _touch[], int _key);							// native部稼働


	/**********
	    開始
	 **********/
	@Override
	protected void	onCreate(Bundle _savedInstanceState)
	{
		onCreate2(_savedInstanceState, null);
	}

	protected void	onCreate2(Bundle _savedInstanceState, FrameLayout _base)
	{
		super.onCreate(_savedInstanceState);

		if ( _savedInstanceState == null ) {
			phase = 0;
			orientation = getResources().getConfiguration().orientation;
			SoundManager.init();										// サウンド管理初期化
		}
		else if ( orientation == getResources().getConfiguration().orientation ) {
			phase = 1;
		}
		else {
			phase = 100;
			return;
		}

		if ( _base == null ) {
			base_layout	= new FrameLayout(this);						// ベースレイアウト
			setContentView(base_layout);
		}
		else {
			base_layout	= _base;
		}

		renderer = new BaseRenderer();									// レンダラー作成
		if ( orientation == Configuration.ORIENTATION_PORTRAIT ) {
			surface_view = new BaseView(getApplication(), renderer);	// ビュー作成
			base_layout.addView(surface_view, 0);
		}
		executor = Executors.newSingleThreadScheduledExecutor();		// 定期実行管理
	}

	/**********
	    終了
	 **********/
	@Override
	protected void	onDestroy()
	{
		if ( phase < 20 ) {
			quitNative();						// native部終了
		}
		if ( executor != null ) {
			executor.shutdown();
			executor = null;
		}
		super.onDestroy();
	}

	@Override
	protected void	onSaveInstanceState(Bundle outState)
	{
		if ( phase < 20 ) {
			phase = 20;
		}
	}

	/**************
	    一時停止
	 **************/
	@Override
	protected void	onPause()
	{
		synchronized (renderer)
		{
			super.onPause();
		}
		if ( receiver != null ) {
			unregisterReceiver(receiver);		// レシーバー登録を解除
			receiver = null;
		}
		if ( phase < 10 ) {
			phase = 10;
			if ( future != null ) {				// 定期実行停止
				while ( phase == 10 ) {			// ポーズ処理待ち
					try {
						Thread.sleep(1);
					}
					catch(Exception ex){}
				}
				future.cancel(false);
				future = null;
				synchronized (renderer) {}
			}
			surface_view.onPause();
			if ( orientation == Configuration.ORIENTATION_LANDSCAPE ) {
				surface_view = null;
			}
		}
		getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);		// スリープ禁止解除
	}

	/**********
	    再開
	 **********/
	@Override
	protected void	onResume()
	{
		super.onResume();
		if ( phase < 100 ) {
			getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);	// スリープ禁止
			for (int i = 0; i < 5; i++) {			// タッチパネル状態クリア
				touch_status[i*3] = 0;
			}
			if ( ((KeyguardManager)getSystemService(Context.KEYGUARD_SERVICE)).inKeyguardRestrictedInputMode() ) {
				receiver = new UnLockReceiver();
				registerReceiver(receiver, new IntentFilter(Intent.ACTION_USER_PRESENT));			// スクリーンロック解除待ち
			}
			else if ( orientation == Configuration.ORIENTATION_PORTRAIT ) {
				surface_view.onResume();
			}
			else {
				surface_view = new BaseView(getApplication(), renderer);			// ビュー作成
				base_layout.addView(surface_view, 0);
			}
		}
	}

	private UnLockReceiver	receiver = null;						// スクリーンロック解除検知用

	private class UnLockReceiver extends BroadcastReceiver
	{
		@Override
		public void		onReceive(Context context, Intent intent)
		{
			unregisterReceiver(receiver);							// レシーバー登録を解除
			receiver = null;
			if ( orientation == Configuration.ORIENTATION_PORTRAIT ) {
				surface_view.onResume();
			}
			else {
				surface_view = new BaseView(getApplication(), renderer);			// ビュー作成
				base_layout.addView(surface_view, 0);
			}
		}
	}


	/********************
	    バックキー入力
	 ********************/
	@Override
	public void		onBackPressed()
	{
		key_status = KEY_BACK;
	}


	/************
	    ビュー
	 ************/
	class BaseView extends GLSurfaceView
	{
		public BaseView(Context _context, BaseRenderer _renderer)
		{
			super(_context);
			setEGLContextClientVersion(2);					// OpenGL ES 2.0使用
			setEGLConfigChooser(new ConfigChooser());
			setRenderer(_renderer);
			setRenderMode(RENDERMODE_WHEN_DIRTY);
		}

		class ConfigChooser implements GLSurfaceView.EGLConfigChooser
		{
			@Override
			public EGLConfig	chooseConfig(EGL10 egl, EGLDisplay display)
			{
				final
				int[]	configAttribs =
				{
					EGL10.EGL_RED_SIZE,		8,
					EGL10.EGL_GREEN_SIZE,	8,
					EGL10.EGL_BLUE_SIZE,	8,
					EGL10.EGL_NONE
				};

				int[]	num_config = new int[1];

				egl.eglChooseConfig(display, configAttribs, null, 0, num_config);
				if ( num_config[0] <= 0 ) {
					return	null;
				}

				EGLConfig[]		configs = new EGLConfig[num_config[0]];

				egl.eglChooseConfig(display, configAttribs, configs, num_config[0], num_config);

				return	configs[0];
			}
		}

		/********************
			タッチイベント
		 ********************/
		public boolean	onTouchEvent(final MotionEvent event)
		{
			if ( future == null ) {
				return	false;
			}

			int		_action = event.getAction();
			int		_index, _id;

			switch ( _action & MotionEvent.ACTION_MASK ) {
			  case MotionEvent.ACTION_DOWN :
			  case MotionEvent.ACTION_POINTER_DOWN :
				_index	= (_action & MotionEvent.ACTION_POINTER_INDEX_MASK) >> MotionEvent.ACTION_POINTER_INDEX_SHIFT;
				_id		= event.getPointerId(_index)*3;
				if ( _id < 5*3 ) {
					touch_status[_id + 1] = (short)event.getX(_index);		// X座標
					touch_status[_id + 2] = (short)event.getY(_index);		// Y座標
					touch_status[_id + 0] = 1;								// タッチ中
				}
				break;

			  case MotionEvent.ACTION_MOVE :
				for (_index = 0; _index < event.getPointerCount(); _index++) {
					_id = event.getPointerId(_index)*3;
					if ( _id < 5*3 ) {
						touch_status[_id + 1] = (short)event.getX(_index);	// X座標
						touch_status[_id + 2] = (short)event.getY(_index);	// Y座標
						touch_status[_id + 0] = 1;							// タッチ中
					}
				}
				break;

			  case MotionEvent.ACTION_UP :
			  case MotionEvent.ACTION_POINTER_UP :
				_index	= (_action & MotionEvent.ACTION_POINTER_INDEX_MASK) >> MotionEvent.ACTION_POINTER_INDEX_SHIFT;
				_id		= event.getPointerId(_index)*3;
				if ( _id < 5*3 ) {
					touch_status[_id + 0] = 0;								// 非タッチ
				}
				break;
			}
			return true;
		}
	}


	/****************
	    レンダラー
	 ****************/
	class BaseRenderer implements GLSurfaceView.Renderer
	{
		public void	onSurfaceCreated(GL10 _gl, EGLConfig _config) {}

		public void	onSurfaceChanged(GL10 gl, int width, int height)
		{
			screen_width  = width;
			screen_height = height;
			synchronized (renderer)
			{
				if ( (phase != 2) && (future == null) ) {
					int		period = initNative((phase == 0) ? true : false, width, height, getAssets());		// native部初期化

					phase      = 2;
					key_status = 0;
					future = executor.scheduleAtFixedRate(new Runnable()	// 定期実行開始
					{
						@Override
						public void	run()
						{
							surface_view.requestRender();		// 描画リクエスト
						}
					},
					0, period, TimeUnit.MILLISECONDS);
				}
			}
		}

		public void	onDrawFrame(GL10 gl)
		{
			synchronized (renderer)
			{
				if ( future != null ) {
					switch ( phase ) {
					  case 2 :					// 稼働中
						{
							int		_key = key_status;

							key_status = 0;
							if ( !updateNative(touch_status, _key) ) {		// native部稼働
								finish();
								break;
							}
							else if ( phase != 10 ) {
								break;
							}
						}

					  case 10 :					// 一時停止
						pauseNative();										// native部一時停止
						phase++;
						break;
					}
				}
			}
		}
	}
}

/***************** End of File ***************************************************/